# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from reducto import Reducto, AsyncReducto
from tests.utils import assert_matches_type
from reducto.types import (
    ParseRunResponse,
    AsyncParseResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestParse:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_overload_1(self, client: Reducto) -> None:
        parse = client.parse.run(
            input="string",
        )
        assert_matches_type(ParseRunResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_with_all_params_overload_1(self, client: Reducto) -> None:
        parse = client.parse.run(
            input="string",
            enhance={
                "agentic": [
                    {
                        "scope": "table",
                        "prompt": "prompt",
                    }
                ],
                "intelligent_ordering": True,
                "summarize_figures": True,
            },
            formatting={
                "add_page_markers": True,
                "include": ["change_tracking"],
                "merge_tables": True,
                "table_output_format": "html",
            },
            retrieval={
                "chunking": {
                    "chunk_mode": "variable",
                    "chunk_overlap": 0,
                    "chunk_size": 0,
                },
                "embedding_optimized": True,
                "filter_blocks": ["Header"],
            },
            settings={
                "document_password": "document_password",
                "embed_pdf_metadata": True,
                "extraction_mode": "ocr",
                "force_file_extension": "force_file_extension",
                "force_url_result": True,
                "ocr_system": "standard",
                "page_range": {
                    "end": 0,
                    "start": 0,
                },
                "persist_results": True,
                "return_images": ["figure"],
                "return_ocr_data": True,
                "timeout": 0,
            },
            spreadsheet={
                "clustering": "accurate",
                "exclude": ["hidden_sheets"],
                "include": ["cell_colors"],
                "split_large_tables": {
                    "enabled": True,
                    "size": 0,
                },
            },
        )
        assert_matches_type(ParseRunResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_run_overload_1(self, client: Reducto) -> None:
        response = client.parse.with_raw_response.run(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        parse = response.parse()
        assert_matches_type(ParseRunResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_run_overload_1(self, client: Reducto) -> None:
        with client.parse.with_streaming_response.run(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            parse = response.parse()
            assert_matches_type(ParseRunResponse, parse, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_overload_2(self, client: Reducto) -> None:
        parse = client.parse.run(
            input="string",
        )
        assert_matches_type(ParseRunResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_with_all_params_overload_2(self, client: Reducto) -> None:
        parse = client.parse.run(
            input="string",
            async_={
                "metadata": {},
                "priority": True,
                "webhook": {
                    "channels": ["string"],
                    "mode": "svix",
                },
            },
            enhance={
                "agentic": [
                    {
                        "scope": "table",
                        "prompt": "prompt",
                    }
                ],
                "intelligent_ordering": True,
                "summarize_figures": True,
            },
            formatting={
                "add_page_markers": True,
                "include": ["change_tracking"],
                "merge_tables": True,
                "table_output_format": "html",
            },
            queue_priority="auto",
            retrieval={
                "chunking": {
                    "chunk_mode": "variable",
                    "chunk_overlap": 0,
                    "chunk_size": 0,
                },
                "embedding_optimized": True,
                "filter_blocks": ["Header"],
            },
            settings={
                "document_password": "document_password",
                "embed_pdf_metadata": True,
                "extraction_mode": "ocr",
                "force_file_extension": "force_file_extension",
                "force_url_result": True,
                "ocr_system": "standard",
                "page_range": {
                    "end": 0,
                    "start": 0,
                },
                "persist_results": True,
                "return_images": ["figure"],
                "return_ocr_data": True,
                "timeout": 0,
            },
            spreadsheet={
                "clustering": "accurate",
                "exclude": ["hidden_sheets"],
                "include": ["cell_colors"],
                "split_large_tables": {
                    "enabled": True,
                    "size": 0,
                },
            },
        )
        assert_matches_type(ParseRunResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_run_overload_2(self, client: Reducto) -> None:
        response = client.parse.with_raw_response.run(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        parse = response.parse()
        assert_matches_type(ParseRunResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_run_overload_2(self, client: Reducto) -> None:
        with client.parse.with_streaming_response.run(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            parse = response.parse()
            assert_matches_type(ParseRunResponse, parse, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_job(self, client: Reducto) -> None:
        parse = client.parse.run_job(
            input="string",
        )
        assert_matches_type(AsyncParseResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_job_with_all_params(self, client: Reducto) -> None:
        parse = client.parse.run_job(
            input="string",
            async_={
                "metadata": {},
                "priority": True,
                "webhook": {
                    "channels": ["string"],
                    "mode": "svix",
                },
            },
            enhance={
                "agentic": [
                    {
                        "scope": "table",
                        "prompt": "prompt",
                    }
                ],
                "intelligent_ordering": True,
                "summarize_figures": True,
            },
            formatting={
                "add_page_markers": True,
                "include": ["change_tracking"],
                "merge_tables": True,
                "table_output_format": "html",
            },
            queue_priority="auto",
            retrieval={
                "chunking": {
                    "chunk_mode": "variable",
                    "chunk_overlap": 0,
                    "chunk_size": 0,
                },
                "embedding_optimized": True,
                "filter_blocks": ["Header"],
            },
            settings={
                "document_password": "document_password",
                "embed_pdf_metadata": True,
                "extraction_mode": "ocr",
                "force_file_extension": "force_file_extension",
                "force_url_result": True,
                "ocr_system": "standard",
                "page_range": {
                    "end": 0,
                    "start": 0,
                },
                "persist_results": True,
                "return_images": ["figure"],
                "return_ocr_data": True,
                "timeout": 0,
            },
            spreadsheet={
                "clustering": "accurate",
                "exclude": ["hidden_sheets"],
                "include": ["cell_colors"],
                "split_large_tables": {
                    "enabled": True,
                    "size": 0,
                },
            },
        )
        assert_matches_type(AsyncParseResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_run_job(self, client: Reducto) -> None:
        response = client.parse.with_raw_response.run_job(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        parse = response.parse()
        assert_matches_type(AsyncParseResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_run_job(self, client: Reducto) -> None:
        with client.parse.with_streaming_response.run_job(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            parse = response.parse()
            assert_matches_type(AsyncParseResponse, parse, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncParse:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_overload_1(self, async_client: AsyncReducto) -> None:
        parse = await async_client.parse.run(
            input="string",
        )
        assert_matches_type(ParseRunResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_with_all_params_overload_1(self, async_client: AsyncReducto) -> None:
        parse = await async_client.parse.run(
            input="string",
            enhance={
                "agentic": [
                    {
                        "scope": "table",
                        "prompt": "prompt",
                    }
                ],
                "intelligent_ordering": True,
                "summarize_figures": True,
            },
            formatting={
                "add_page_markers": True,
                "include": ["change_tracking"],
                "merge_tables": True,
                "table_output_format": "html",
            },
            retrieval={
                "chunking": {
                    "chunk_mode": "variable",
                    "chunk_overlap": 0,
                    "chunk_size": 0,
                },
                "embedding_optimized": True,
                "filter_blocks": ["Header"],
            },
            settings={
                "document_password": "document_password",
                "embed_pdf_metadata": True,
                "extraction_mode": "ocr",
                "force_file_extension": "force_file_extension",
                "force_url_result": True,
                "ocr_system": "standard",
                "page_range": {
                    "end": 0,
                    "start": 0,
                },
                "persist_results": True,
                "return_images": ["figure"],
                "return_ocr_data": True,
                "timeout": 0,
            },
            spreadsheet={
                "clustering": "accurate",
                "exclude": ["hidden_sheets"],
                "include": ["cell_colors"],
                "split_large_tables": {
                    "enabled": True,
                    "size": 0,
                },
            },
        )
        assert_matches_type(ParseRunResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_run_overload_1(self, async_client: AsyncReducto) -> None:
        response = await async_client.parse.with_raw_response.run(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        parse = await response.parse()
        assert_matches_type(ParseRunResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_run_overload_1(self, async_client: AsyncReducto) -> None:
        async with async_client.parse.with_streaming_response.run(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            parse = await response.parse()
            assert_matches_type(ParseRunResponse, parse, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_overload_2(self, async_client: AsyncReducto) -> None:
        parse = await async_client.parse.run(
            input="string",
        )
        assert_matches_type(ParseRunResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_with_all_params_overload_2(self, async_client: AsyncReducto) -> None:
        parse = await async_client.parse.run(
            input="string",
            async_={
                "metadata": {},
                "priority": True,
                "webhook": {
                    "channels": ["string"],
                    "mode": "svix",
                },
            },
            enhance={
                "agentic": [
                    {
                        "scope": "table",
                        "prompt": "prompt",
                    }
                ],
                "intelligent_ordering": True,
                "summarize_figures": True,
            },
            formatting={
                "add_page_markers": True,
                "include": ["change_tracking"],
                "merge_tables": True,
                "table_output_format": "html",
            },
            queue_priority="auto",
            retrieval={
                "chunking": {
                    "chunk_mode": "variable",
                    "chunk_overlap": 0,
                    "chunk_size": 0,
                },
                "embedding_optimized": True,
                "filter_blocks": ["Header"],
            },
            settings={
                "document_password": "document_password",
                "embed_pdf_metadata": True,
                "extraction_mode": "ocr",
                "force_file_extension": "force_file_extension",
                "force_url_result": True,
                "ocr_system": "standard",
                "page_range": {
                    "end": 0,
                    "start": 0,
                },
                "persist_results": True,
                "return_images": ["figure"],
                "return_ocr_data": True,
                "timeout": 0,
            },
            spreadsheet={
                "clustering": "accurate",
                "exclude": ["hidden_sheets"],
                "include": ["cell_colors"],
                "split_large_tables": {
                    "enabled": True,
                    "size": 0,
                },
            },
        )
        assert_matches_type(ParseRunResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_run_overload_2(self, async_client: AsyncReducto) -> None:
        response = await async_client.parse.with_raw_response.run(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        parse = await response.parse()
        assert_matches_type(ParseRunResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_run_overload_2(self, async_client: AsyncReducto) -> None:
        async with async_client.parse.with_streaming_response.run(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            parse = await response.parse()
            assert_matches_type(ParseRunResponse, parse, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_job(self, async_client: AsyncReducto) -> None:
        parse = await async_client.parse.run_job(
            input="string",
        )
        assert_matches_type(AsyncParseResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_job_with_all_params(self, async_client: AsyncReducto) -> None:
        parse = await async_client.parse.run_job(
            input="string",
            async_={
                "metadata": {},
                "priority": True,
                "webhook": {
                    "channels": ["string"],
                    "mode": "svix",
                },
            },
            enhance={
                "agentic": [
                    {
                        "scope": "table",
                        "prompt": "prompt",
                    }
                ],
                "intelligent_ordering": True,
                "summarize_figures": True,
            },
            formatting={
                "add_page_markers": True,
                "include": ["change_tracking"],
                "merge_tables": True,
                "table_output_format": "html",
            },
            queue_priority="auto",
            retrieval={
                "chunking": {
                    "chunk_mode": "variable",
                    "chunk_overlap": 0,
                    "chunk_size": 0,
                },
                "embedding_optimized": True,
                "filter_blocks": ["Header"],
            },
            settings={
                "document_password": "document_password",
                "embed_pdf_metadata": True,
                "extraction_mode": "ocr",
                "force_file_extension": "force_file_extension",
                "force_url_result": True,
                "ocr_system": "standard",
                "page_range": {
                    "end": 0,
                    "start": 0,
                },
                "persist_results": True,
                "return_images": ["figure"],
                "return_ocr_data": True,
                "timeout": 0,
            },
            spreadsheet={
                "clustering": "accurate",
                "exclude": ["hidden_sheets"],
                "include": ["cell_colors"],
                "split_large_tables": {
                    "enabled": True,
                    "size": 0,
                },
            },
        )
        assert_matches_type(AsyncParseResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_run_job(self, async_client: AsyncReducto) -> None:
        response = await async_client.parse.with_raw_response.run_job(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        parse = await response.parse()
        assert_matches_type(AsyncParseResponse, parse, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_run_job(self, async_client: AsyncReducto) -> None:
        async with async_client.parse.with_streaming_response.run_job(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            parse = await response.parse()
            assert_matches_type(AsyncParseResponse, parse, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["FigureAgentic"]


class FigureAgentic(TypedDict, total=False):
    scope: Required[Literal["figure"]]

    advanced_chart_agent: bool
    """If True, use the advanced chart agent. Defaults to False."""

    prompt: Optional[str]
    """Custom prompt for figure agentic."""

    return_overlays: bool
    """If True, return overlays for the figure.

    This is so you can use the overlays to double check the quality of the
    extraction
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["TableAgentic"]


class TableAgentic(TypedDict, total=False):
    scope: Required[Literal["table"]]

    prompt: Optional[str]
    """Custom prompt for table agentic."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["Formatting"]


class Formatting(TypedDict, total=False):
    add_page_markers: bool
    """If True, add page markers to the output.

    Defaults to False. Useful for extracting data with page specific information.
    """

    include: List[Literal["change_tracking", "highlight", "comments", "hyperlinks", "signatures", "ignore_watermarks"]]
    """A list of formatting to include in the output."""

    merge_tables: bool
    """
    A flag to indicate if consecutive tables with the same number of columns should
    be merged. Defaults to False.
    """

    table_output_format: Literal["html", "json", "md", "jsonbbox", "dynamic", "csv"]
    """The mode to use for table output.

    Defaults to dynamic, which returns md for simpler tables and html for more
    complex tables.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Dict, Union, Mapping, Optional, cast
from typing_extensions import Self, Literal, override

import httpx

from . import _exceptions
from ._qs import Querystring
from .types import client_upload_params
from ._types import (
    Body,
    Omit,
    Query,
    Headers,
    Timeout,
    NotGiven,
    FileTypes,
    Transport,
    ProxiesTypes,
    RequestOptions,
    omit,
    not_given,
)
from ._utils import (
    is_given,
    extract_files,
    maybe_transform,
    deepcopy_minimal,
    get_async_library,
    async_maybe_transform,
)
from ._compat import cached_property
from ._models import SecurityOptions
from ._version import __version__
from ._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import ReductoError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
    make_request_options,
)
from .types.shared.upload import Upload

if TYPE_CHECKING:
    from .resources import job, edit, parse, split, extract, webhook, classify, pipeline
    from .resources.job import JobResource, AsyncJobResource
    from .resources.edit import EditResource, AsyncEditResource
    from .resources.parse import ParseResource, AsyncParseResource
    from .resources.split import SplitResource, AsyncSplitResource
    from .resources.extract import ExtractResource, AsyncExtractResource
    from .resources.webhook import WebhookResource, AsyncWebhookResource
    from .resources.classify import ClassifyResource, AsyncClassifyResource
    from .resources.pipeline import PipelineResource, AsyncPipelineResource

__all__ = [
    "ENVIRONMENTS",
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Reducto",
    "AsyncReducto",
    "Client",
    "AsyncClient",
]

ENVIRONMENTS: Dict[str, str] = {
    "production": "https://platform.reducto.ai",
    "eu": "https://eu.platform.reducto.ai",
    "au": "https://au.platform.reducto.ai",
}


class Reducto(SyncAPIClient):
    # client options
    api_key: str

    _environment: Literal["production", "eu", "au"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        environment: Literal["production", "eu", "au"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Reducto client instance.

        This automatically infers the `api_key` argument from the `REDUCTO_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("REDUCTO_API_KEY")
        if api_key is None:
            raise ReductoError(
                "The api_key client option must be set either by passing api_key to the client or by setting the REDUCTO_API_KEY environment variable"
            )
        self.api_key = api_key

        self._environment = environment

        base_url_env = os.environ.get("REDUCTO_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `REDUCTO_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "production"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def parse(self) -> ParseResource:
        from .resources.parse import ParseResource

        return ParseResource(self)

    @cached_property
    def extract(self) -> ExtractResource:
        from .resources.extract import ExtractResource

        return ExtractResource(self)

    @cached_property
    def split(self) -> SplitResource:
        from .resources.split import SplitResource

        return SplitResource(self)

    @cached_property
    def edit(self) -> EditResource:
        from .resources.edit import EditResource

        return EditResource(self)

    @cached_property
    def pipeline(self) -> PipelineResource:
        from .resources.pipeline import PipelineResource

        return PipelineResource(self)

    @cached_property
    def classify(self) -> ClassifyResource:
        from .resources.classify import ClassifyResource

        return ClassifyResource(self)

    @cached_property
    def webhook(self) -> WebhookResource:
        from .resources.webhook import WebhookResource

        return WebhookResource(self)

    @cached_property
    def job(self) -> JobResource:
        from .resources.job import JobResource

        return JobResource(self)

    @cached_property
    def with_raw_response(self) -> ReductoWithRawResponse:
        return ReductoWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReductoWithStreamedResponse:
        return ReductoWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @override
    def _auth_headers(self, security: SecurityOptions) -> dict[str, str]:
        return {
            **(self._skippable_http_bearer if security.get("skippable_http_bearer", False) else {}),
        }

    @property
    def _skippable_http_bearer(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        environment: Literal["production", "eu", "au"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    def api_version(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Get Version"""
        return self.get(
            "/version",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )

    def upload(
        self,
        *,
        extension: Optional[str] | Omit = omit,
        file: Union[FileTypes, str, None] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Upload:
        """
        Upload

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal({})
        if file is not omit:
            body["file"] = file
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        if files:
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self.post(
            "/upload",
            body=maybe_transform(body, client_upload_params.ClientUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"extension": extension}, client_upload_params.ClientUploadParams),
            ),
            cast_to=Upload,
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncReducto(AsyncAPIClient):
    # client options
    api_key: str

    _environment: Literal["production", "eu", "au"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        environment: Literal["production", "eu", "au"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncReducto client instance.

        This automatically infers the `api_key` argument from the `REDUCTO_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("REDUCTO_API_KEY")
        if api_key is None:
            raise ReductoError(
                "The api_key client option must be set either by passing api_key to the client or by setting the REDUCTO_API_KEY environment variable"
            )
        self.api_key = api_key

        self._environment = environment

        base_url_env = os.environ.get("REDUCTO_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `REDUCTO_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "production"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def parse(self) -> AsyncParseResource:
        from .resources.parse import AsyncParseResource

        return AsyncParseResource(self)

    @cached_property
    def extract(self) -> AsyncExtractResource:
        from .resources.extract import AsyncExtractResource

        return AsyncExtractResource(self)

    @cached_property
    def split(self) -> AsyncSplitResource:
        from .resources.split import AsyncSplitResource

        return AsyncSplitResource(self)

    @cached_property
    def edit(self) -> AsyncEditResource:
        from .resources.edit import AsyncEditResource

        return AsyncEditResource(self)

    @cached_property
    def pipeline(self) -> AsyncPipelineResource:
        from .resources.pipeline import AsyncPipelineResource

        return AsyncPipelineResource(self)

    @cached_property
    def classify(self) -> AsyncClassifyResource:
        from .resources.classify import AsyncClassifyResource

        return AsyncClassifyResource(self)

    @cached_property
    def webhook(self) -> AsyncWebhookResource:
        from .resources.webhook import AsyncWebhookResource

        return AsyncWebhookResource(self)

    @cached_property
    def job(self) -> AsyncJobResource:
        from .resources.job import AsyncJobResource

        return AsyncJobResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncReductoWithRawResponse:
        return AsyncReductoWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReductoWithStreamedResponse:
        return AsyncReductoWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @override
    def _auth_headers(self, security: SecurityOptions) -> dict[str, str]:
        return {
            **(self._skippable_http_bearer if security.get("skippable_http_bearer", False) else {}),
        }

    @property
    def _skippable_http_bearer(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        environment: Literal["production", "eu", "au"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    async def api_version(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Get Version"""
        return await self.get(
            "/version",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )

    async def upload(
        self,
        *,
        extension: Optional[str] | Omit = omit,
        file: Union[FileTypes, str, None] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Upload:
        """
        Upload

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal({})
        if file is not omit:
            body["file"] = file
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        if files:
            extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self.post(
            "/upload",
            body=await async_maybe_transform(body, client_upload_params.ClientUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"extension": extension}, client_upload_params.ClientUploadParams),
            ),
            cast_to=Upload,
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class ReductoWithRawResponse:
    _client: Reducto

    def __init__(self, client: Reducto) -> None:
        self._client = client

        self.api_version = to_raw_response_wrapper(
            client.api_version,
        )
        self.upload = to_raw_response_wrapper(
            client.upload,
        )

    @cached_property
    def parse(self) -> parse.ParseResourceWithRawResponse:
        from .resources.parse import ParseResourceWithRawResponse

        return ParseResourceWithRawResponse(self._client.parse)

    @cached_property
    def extract(self) -> extract.ExtractResourceWithRawResponse:
        from .resources.extract import ExtractResourceWithRawResponse

        return ExtractResourceWithRawResponse(self._client.extract)

    @cached_property
    def split(self) -> split.SplitResourceWithRawResponse:
        from .resources.split import SplitResourceWithRawResponse

        return SplitResourceWithRawResponse(self._client.split)

    @cached_property
    def edit(self) -> edit.EditResourceWithRawResponse:
        from .resources.edit import EditResourceWithRawResponse

        return EditResourceWithRawResponse(self._client.edit)

    @cached_property
    def pipeline(self) -> pipeline.PipelineResourceWithRawResponse:
        from .resources.pipeline import PipelineResourceWithRawResponse

        return PipelineResourceWithRawResponse(self._client.pipeline)

    @cached_property
    def classify(self) -> classify.ClassifyResourceWithRawResponse:
        from .resources.classify import ClassifyResourceWithRawResponse

        return ClassifyResourceWithRawResponse(self._client.classify)

    @cached_property
    def webhook(self) -> webhook.WebhookResourceWithRawResponse:
        from .resources.webhook import WebhookResourceWithRawResponse

        return WebhookResourceWithRawResponse(self._client.webhook)

    @cached_property
    def job(self) -> job.JobResourceWithRawResponse:
        from .resources.job import JobResourceWithRawResponse

        return JobResourceWithRawResponse(self._client.job)


class AsyncReductoWithRawResponse:
    _client: AsyncReducto

    def __init__(self, client: AsyncReducto) -> None:
        self._client = client

        self.api_version = async_to_raw_response_wrapper(
            client.api_version,
        )
        self.upload = async_to_raw_response_wrapper(
            client.upload,
        )

    @cached_property
    def parse(self) -> parse.AsyncParseResourceWithRawResponse:
        from .resources.parse import AsyncParseResourceWithRawResponse

        return AsyncParseResourceWithRawResponse(self._client.parse)

    @cached_property
    def extract(self) -> extract.AsyncExtractResourceWithRawResponse:
        from .resources.extract import AsyncExtractResourceWithRawResponse

        return AsyncExtractResourceWithRawResponse(self._client.extract)

    @cached_property
    def split(self) -> split.AsyncSplitResourceWithRawResponse:
        from .resources.split import AsyncSplitResourceWithRawResponse

        return AsyncSplitResourceWithRawResponse(self._client.split)

    @cached_property
    def edit(self) -> edit.AsyncEditResourceWithRawResponse:
        from .resources.edit import AsyncEditResourceWithRawResponse

        return AsyncEditResourceWithRawResponse(self._client.edit)

    @cached_property
    def pipeline(self) -> pipeline.AsyncPipelineResourceWithRawResponse:
        from .resources.pipeline import AsyncPipelineResourceWithRawResponse

        return AsyncPipelineResourceWithRawResponse(self._client.pipeline)

    @cached_property
    def classify(self) -> classify.AsyncClassifyResourceWithRawResponse:
        from .resources.classify import AsyncClassifyResourceWithRawResponse

        return AsyncClassifyResourceWithRawResponse(self._client.classify)

    @cached_property
    def webhook(self) -> webhook.AsyncWebhookResourceWithRawResponse:
        from .resources.webhook import AsyncWebhookResourceWithRawResponse

        return AsyncWebhookResourceWithRawResponse(self._client.webhook)

    @cached_property
    def job(self) -> job.AsyncJobResourceWithRawResponse:
        from .resources.job import AsyncJobResourceWithRawResponse

        return AsyncJobResourceWithRawResponse(self._client.job)


class ReductoWithStreamedResponse:
    _client: Reducto

    def __init__(self, client: Reducto) -> None:
        self._client = client

        self.api_version = to_streamed_response_wrapper(
            client.api_version,
        )
        self.upload = to_streamed_response_wrapper(
            client.upload,
        )

    @cached_property
    def parse(self) -> parse.ParseResourceWithStreamingResponse:
        from .resources.parse import ParseResourceWithStreamingResponse

        return ParseResourceWithStreamingResponse(self._client.parse)

    @cached_property
    def extract(self) -> extract.ExtractResourceWithStreamingResponse:
        from .resources.extract import ExtractResourceWithStreamingResponse

        return ExtractResourceWithStreamingResponse(self._client.extract)

    @cached_property
    def split(self) -> split.SplitResourceWithStreamingResponse:
        from .resources.split import SplitResourceWithStreamingResponse

        return SplitResourceWithStreamingResponse(self._client.split)

    @cached_property
    def edit(self) -> edit.EditResourceWithStreamingResponse:
        from .resources.edit import EditResourceWithStreamingResponse

        return EditResourceWithStreamingResponse(self._client.edit)

    @cached_property
    def pipeline(self) -> pipeline.PipelineResourceWithStreamingResponse:
        from .resources.pipeline import PipelineResourceWithStreamingResponse

        return PipelineResourceWithStreamingResponse(self._client.pipeline)

    @cached_property
    def classify(self) -> classify.ClassifyResourceWithStreamingResponse:
        from .resources.classify import ClassifyResourceWithStreamingResponse

        return ClassifyResourceWithStreamingResponse(self._client.classify)

    @cached_property
    def webhook(self) -> webhook.WebhookResourceWithStreamingResponse:
        from .resources.webhook import WebhookResourceWithStreamingResponse

        return WebhookResourceWithStreamingResponse(self._client.webhook)

    @cached_property
    def job(self) -> job.JobResourceWithStreamingResponse:
        from .resources.job import JobResourceWithStreamingResponse

        return JobResourceWithStreamingResponse(self._client.job)


class AsyncReductoWithStreamedResponse:
    _client: AsyncReducto

    def __init__(self, client: AsyncReducto) -> None:
        self._client = client

        self.api_version = async_to_streamed_response_wrapper(
            client.api_version,
        )
        self.upload = async_to_streamed_response_wrapper(
            client.upload,
        )

    @cached_property
    def parse(self) -> parse.AsyncParseResourceWithStreamingResponse:
        from .resources.parse import AsyncParseResourceWithStreamingResponse

        return AsyncParseResourceWithStreamingResponse(self._client.parse)

    @cached_property
    def extract(self) -> extract.AsyncExtractResourceWithStreamingResponse:
        from .resources.extract import AsyncExtractResourceWithStreamingResponse

        return AsyncExtractResourceWithStreamingResponse(self._client.extract)

    @cached_property
    def split(self) -> split.AsyncSplitResourceWithStreamingResponse:
        from .resources.split import AsyncSplitResourceWithStreamingResponse

        return AsyncSplitResourceWithStreamingResponse(self._client.split)

    @cached_property
    def edit(self) -> edit.AsyncEditResourceWithStreamingResponse:
        from .resources.edit import AsyncEditResourceWithStreamingResponse

        return AsyncEditResourceWithStreamingResponse(self._client.edit)

    @cached_property
    def pipeline(self) -> pipeline.AsyncPipelineResourceWithStreamingResponse:
        from .resources.pipeline import AsyncPipelineResourceWithStreamingResponse

        return AsyncPipelineResourceWithStreamingResponse(self._client.pipeline)

    @cached_property
    def classify(self) -> classify.AsyncClassifyResourceWithStreamingResponse:
        from .resources.classify import AsyncClassifyResourceWithStreamingResponse

        return AsyncClassifyResourceWithStreamingResponse(self._client.classify)

    @cached_property
    def webhook(self) -> webhook.AsyncWebhookResourceWithStreamingResponse:
        from .resources.webhook import AsyncWebhookResourceWithStreamingResponse

        return AsyncWebhookResourceWithStreamingResponse(self._client.webhook)

    @cached_property
    def job(self) -> job.AsyncJobResourceWithStreamingResponse:
        from .resources.job import AsyncJobResourceWithStreamingResponse

        return AsyncJobResourceWithStreamingResponse(self._client.job)


Client = Reducto

AsyncClient = AsyncReducto



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, cast
from typing_extensions import overload

import httpx

from ..types import (
    extract_run_params,
    extract_run_job_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import required_args, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.instructions_param import InstructionsParam
from ..types.parse_options_param import ParseOptionsParam
from ..types.extract_run_response import ExtractRunResponse
from ..types.async_config_v3_param import AsyncConfigV3Param
from ..types.async_extract_response import AsyncExtractResponse
from ..types.extract_settings_param import ExtractSettingsParam

__all__ = ["ExtractResource", "AsyncExtractResource"]


class ExtractResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ExtractResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ExtractResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExtractResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#with_streaming_response
        """
        return ExtractResourceWithStreamingResponse(self)

    @overload
    def run(
        self,
        *,
        input: extract_run_params.SyncExtractConfigInput,
        instructions: InstructionsParam | Omit = omit,
        parsing: ParseOptionsParam | Omit = omit,
        settings: ExtractSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExtractRunResponse:
        """
        Extract

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          instructions: The instructions to use for the extraction.

          parsing: The configuration options for parsing the document. If you are passing in a
              jobid:// URL for the file, then this configuration will be ignored.

          settings: The settings to use for the extraction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def run(
        self,
        *,
        input: extract_run_params.AsyncExtractConfigInput,
        async_: AsyncConfigV3Param | Omit = omit,
        instructions: InstructionsParam | Omit = omit,
        parsing: ParseOptionsParam | Omit = omit,
        settings: ExtractSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExtractRunResponse:
        """
        Extract

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          async_: The configuration options for asynchronous processing (default synchronous).

          instructions: The instructions to use for the extraction.

          parsing: The configuration options for parsing the document. If you are passing in a
              jobid:// URL for the file, then this configuration will be ignored.

          settings: The settings to use for the extraction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["input"])
    def run(
        self,
        *,
        input: extract_run_params.SyncExtractConfigInput | extract_run_params.AsyncExtractConfigInput,
        instructions: InstructionsParam | Omit = omit,
        parsing: ParseOptionsParam | Omit = omit,
        settings: ExtractSettingsParam | Omit = omit,
        async_: AsyncConfigV3Param | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExtractRunResponse:
        return cast(
            ExtractRunResponse,
            self._post(
                "/extract",
                body=maybe_transform(
                    {
                        "input": input,
                        "instructions": instructions,
                        "parsing": parsing,
                        "settings": settings,
                        "async_": async_,
                    },
                    extract_run_params.ExtractRunParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ExtractRunResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def run_job(
        self,
        *,
        input: extract_run_job_params.Input,
        async_: AsyncConfigV3Param | Omit = omit,
        instructions: InstructionsParam | Omit = omit,
        parsing: ParseOptionsParam | Omit = omit,
        settings: ExtractSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncExtractResponse:
        """
        Extract Async

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          async_: The configuration options for asynchronous processing (default synchronous).

          instructions: The instructions to use for the extraction.

          parsing: The configuration options for parsing the document. If you are passing in a
              jobid:// URL for the file, then this configuration will be ignored.

          settings: The settings to use for the extraction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/extract_async",
            body=maybe_transform(
                {
                    "input": input,
                    "async_": async_,
                    "instructions": instructions,
                    "parsing": parsing,
                    "settings": settings,
                },
                extract_run_job_params.ExtractRunJobParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncExtractResponse,
        )


class AsyncExtractResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncExtractResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncExtractResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExtractResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#with_streaming_response
        """
        return AsyncExtractResourceWithStreamingResponse(self)

    @overload
    async def run(
        self,
        *,
        input: extract_run_params.SyncExtractConfigInput,
        instructions: InstructionsParam | Omit = omit,
        parsing: ParseOptionsParam | Omit = omit,
        settings: ExtractSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExtractRunResponse:
        """
        Extract

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          instructions: The instructions to use for the extraction.

          parsing: The configuration options for parsing the document. If you are passing in a
              jobid:// URL for the file, then this configuration will be ignored.

          settings: The settings to use for the extraction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def run(
        self,
        *,
        input: extract_run_params.AsyncExtractConfigInput,
        async_: AsyncConfigV3Param | Omit = omit,
        instructions: InstructionsParam | Omit = omit,
        parsing: ParseOptionsParam | Omit = omit,
        settings: ExtractSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExtractRunResponse:
        """
        Extract

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          async_: The configuration options for asynchronous processing (default synchronous).

          instructions: The instructions to use for the extraction.

          parsing: The configuration options for parsing the document. If you are passing in a
              jobid:// URL for the file, then this configuration will be ignored.

          settings: The settings to use for the extraction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["input"])
    async def run(
        self,
        *,
        input: extract_run_params.SyncExtractConfigInput | extract_run_params.AsyncExtractConfigInput,
        instructions: InstructionsParam | Omit = omit,
        parsing: ParseOptionsParam | Omit = omit,
        settings: ExtractSettingsParam | Omit = omit,
        async_: AsyncConfigV3Param | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ExtractRunResponse:
        return cast(
            ExtractRunResponse,
            await self._post(
                "/extract",
                body=await async_maybe_transform(
                    {
                        "input": input,
                        "instructions": instructions,
                        "parsing": parsing,
                        "settings": settings,
                        "async_": async_,
                    },
                    extract_run_params.ExtractRunParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ExtractRunResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def run_job(
        self,
        *,
        input: extract_run_job_params.Input,
        async_: AsyncConfigV3Param | Omit = omit,
        instructions: InstructionsParam | Omit = omit,
        parsing: ParseOptionsParam | Omit = omit,
        settings: ExtractSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncExtractResponse:
        """
        Extract Async

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          async_: The configuration options for asynchronous processing (default synchronous).

          instructions: The instructions to use for the extraction.

          parsing: The configuration options for parsing the document. If you are passing in a
              jobid:// URL for the file, then this configuration will be ignored.

          settings: The settings to use for the extraction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/extract_async",
            body=await async_maybe_transform(
                {
                    "input": input,
                    "async_": async_,
                    "instructions": instructions,
                    "parsing": parsing,
                    "settings": settings,
                },
                extract_run_job_params.ExtractRunJobParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncExtractResponse,
        )


class ExtractResourceWithRawResponse:
    def __init__(self, extract: ExtractResource) -> None:
        self._extract = extract

        self.run = to_raw_response_wrapper(
            extract.run,
        )
        self.run_job = to_raw_response_wrapper(
            extract.run_job,
        )


class AsyncExtractResourceWithRawResponse:
    def __init__(self, extract: AsyncExtractResource) -> None:
        self._extract = extract

        self.run = async_to_raw_response_wrapper(
            extract.run,
        )
        self.run_job = async_to_raw_response_wrapper(
            extract.run_job,
        )


class ExtractResourceWithStreamingResponse:
    def __init__(self, extract: ExtractResource) -> None:
        self._extract = extract

        self.run = to_streamed_response_wrapper(
            extract.run,
        )
        self.run_job = to_streamed_response_wrapper(
            extract.run_job,
        )


class AsyncExtractResourceWithStreamingResponse:
    def __init__(self, extract: AsyncExtractResource) -> None:
        self._extract = extract

        self.run = async_to_streamed_response_wrapper(
            extract.run,
        )
        self.run_job = async_to_streamed_response_wrapper(
            extract.run_job,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .enhance_param import EnhanceParam
from .settings_param import SettingsParam
from .retrieval_param import RetrievalParam
from .formatting_param import FormattingParam
from .spreadsheet_param import SpreadsheetParam

__all__ = ["ParseOptionsParam"]


class ParseOptionsParam(TypedDict, total=False):
    enhance: EnhanceParam

    formatting: FormattingParam

    retrieval: RetrievalParam

    settings: SettingsParam

    spreadsheet: SpreadsheetParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Required, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .page_range_param import PageRangeParam
from .shared_params.upload import Upload

__all__ = ["ClassifyRunParams", "Input", "ClassificationSchema", "PageRange"]


class ClassifyRunParams(TypedDict, total=False):
    input: Required[Input]
    """For parse/split/extract pipelines, the URL of the document to be processed.

    You can provide one of the following: 1. A publicly available URL 2. A presigned
    S3 URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
    directly uploading a document 4. A jobid:// prefixed URL obtained from a
    previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
    API only)

                For edit pipelines, this should be a string containing the edit instructions
    """

    classification_schema: Iterable[ClassificationSchema]
    """A list of classification categories and their matching criteria."""

    document_metadata: Optional[str]
    """Optional document-level metadata to include in classification prompts."""

    page_range: Optional[PageRange]
    """The page range to process (1-indexed).

    By default, the first 5 pages are used. If more than 25 pages are selected, only
    the first 25 (after sorting) are used. Only applies to PDFs; ignored for other
    document types.
    """

    persist_results: bool
    """If True, persist the results indefinitely. Defaults to False."""


Input: TypeAlias = Union[str, SequenceNotStr[str], Upload]


class ClassificationSchema(TypedDict, total=False):
    """A single classification category with its matching criteria."""

    category: Required[str]
    """
    The category name/label that documents will be classified into (e.g., 'invoice',
    'contract', 'receipt').
    """

    criteria: Required[SequenceNotStr[str]]
    """
    A list of criteria, keywords, or descriptions that define what characteristics a
    document must have to be classified into this category (e.g., ['contains billing
    information', 'has itemized charges']).
    """


PageRange: TypeAlias = Union[PageRangeParam, Iterable[PageRangeParam], Iterable[int]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["BoundingBoxParam"]


class BoundingBoxParam(TypedDict, total=False):
    height: Required[float]

    left: Required[float]

    page: Required[int]
    """The page number of the bounding box (1-indexed)."""

    top: Required[float]

    width: Required[float]

    original_page: int
    """The page number in the original document of the bounding box (1-indexed)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional

import httpx

from ..types import edit_run_params, edit_run_job_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.edit_response import EditResponse
from ..types.edit_widget_param import EditWidgetParam
from ..types.edit_options_param import EditOptionsParam
from ..types.edit_run_job_response import EditRunJobResponse

__all__ = ["EditResource", "AsyncEditResource"]


class EditResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EditResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#accessing-raw-response-data-eg-headers
        """
        return EditResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EditResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#with_streaming_response
        """
        return EditResourceWithStreamingResponse(self)

    def run(
        self,
        *,
        document_url: edit_run_params.DocumentURL,
        edit_instructions: str,
        edit_options: EditOptionsParam | Omit = omit,
        form_schema: Optional[Iterable[EditWidgetParam]] | Omit = omit,
        priority: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EditResponse:
        """Edit

        Args:
          document_url:
              The URL of the document to be processed.

        You can provide one of the following:

              1. A publicly available URL
              2. A presigned S3 URL
              3. A reducto:// prefixed URL obtained from the /upload endpoint after directly
                 uploading a document

          edit_instructions: The instructions for the edit.

          form_schema: Form schema for PDF forms. List of widgets with their types, descriptions, and
              bounding boxes. Only works for PDFs.

          priority: If True, attempts to process the job with priority if the user has priority
              processing budget available; by default, sync jobs are prioritized above async
              jobs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/edit",
            body=maybe_transform(
                {
                    "document_url": document_url,
                    "edit_instructions": edit_instructions,
                    "edit_options": edit_options,
                    "form_schema": form_schema,
                    "priority": priority,
                },
                edit_run_params.EditRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EditResponse,
        )

    def run_job(
        self,
        *,
        document_url: edit_run_job_params.DocumentURL,
        edit_instructions: str,
        edit_options: EditOptionsParam | Omit = omit,
        form_schema: Optional[Iterable[EditWidgetParam]] | Omit = omit,
        priority: bool | Omit = omit,
        webhook: edit_run_job_params.Webhook | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EditRunJobResponse:
        """Edit Async

        Args:
          document_url:
              The URL of the document to be processed.

        You can provide one of the following:

              1. A publicly available URL
              2. A presigned S3 URL
              3. A reducto:// prefixed URL obtained from the /upload endpoint after directly
                 uploading a document

          edit_instructions: The instructions for the edit.

          form_schema: Form schema for PDF forms. List of widgets with their types, descriptions, and
              bounding boxes. Only works for PDFs.

          priority: If True, attempts to process the job with priority if the user has priority
              processing budget available; by default, sync jobs are prioritized above async
              jobs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/edit_async",
            body=maybe_transform(
                {
                    "document_url": document_url,
                    "edit_instructions": edit_instructions,
                    "edit_options": edit_options,
                    "form_schema": form_schema,
                    "priority": priority,
                    "webhook": webhook,
                },
                edit_run_job_params.EditRunJobParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EditRunJobResponse,
        )


class AsyncEditResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEditResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncEditResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEditResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#with_streaming_response
        """
        return AsyncEditResourceWithStreamingResponse(self)

    async def run(
        self,
        *,
        document_url: edit_run_params.DocumentURL,
        edit_instructions: str,
        edit_options: EditOptionsParam | Omit = omit,
        form_schema: Optional[Iterable[EditWidgetParam]] | Omit = omit,
        priority: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EditResponse:
        """Edit

        Args:
          document_url:
              The URL of the document to be processed.

        You can provide one of the following:

              1. A publicly available URL
              2. A presigned S3 URL
              3. A reducto:// prefixed URL obtained from the /upload endpoint after directly
                 uploading a document

          edit_instructions: The instructions for the edit.

          form_schema: Form schema for PDF forms. List of widgets with their types, descriptions, and
              bounding boxes. Only works for PDFs.

          priority: If True, attempts to process the job with priority if the user has priority
              processing budget available; by default, sync jobs are prioritized above async
              jobs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/edit",
            body=await async_maybe_transform(
                {
                    "document_url": document_url,
                    "edit_instructions": edit_instructions,
                    "edit_options": edit_options,
                    "form_schema": form_schema,
                    "priority": priority,
                },
                edit_run_params.EditRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EditResponse,
        )

    async def run_job(
        self,
        *,
        document_url: edit_run_job_params.DocumentURL,
        edit_instructions: str,
        edit_options: EditOptionsParam | Omit = omit,
        form_schema: Optional[Iterable[EditWidgetParam]] | Omit = omit,
        priority: bool | Omit = omit,
        webhook: edit_run_job_params.Webhook | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EditRunJobResponse:
        """Edit Async

        Args:
          document_url:
              The URL of the document to be processed.

        You can provide one of the following:

              1. A publicly available URL
              2. A presigned S3 URL
              3. A reducto:// prefixed URL obtained from the /upload endpoint after directly
                 uploading a document

          edit_instructions: The instructions for the edit.

          form_schema: Form schema for PDF forms. List of widgets with their types, descriptions, and
              bounding boxes. Only works for PDFs.

          priority: If True, attempts to process the job with priority if the user has priority
              processing budget available; by default, sync jobs are prioritized above async
              jobs.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/edit_async",
            body=await async_maybe_transform(
                {
                    "document_url": document_url,
                    "edit_instructions": edit_instructions,
                    "edit_options": edit_options,
                    "form_schema": form_schema,
                    "priority": priority,
                    "webhook": webhook,
                },
                edit_run_job_params.EditRunJobParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EditRunJobResponse,
        )


class EditResourceWithRawResponse:
    def __init__(self, edit: EditResource) -> None:
        self._edit = edit

        self.run = to_raw_response_wrapper(
            edit.run,
        )
        self.run_job = to_raw_response_wrapper(
            edit.run_job,
        )


class AsyncEditResourceWithRawResponse:
    def __init__(self, edit: AsyncEditResource) -> None:
        self._edit = edit

        self.run = async_to_raw_response_wrapper(
            edit.run,
        )
        self.run_job = async_to_raw_response_wrapper(
            edit.run_job,
        )


class EditResourceWithStreamingResponse:
    def __init__(self, edit: EditResource) -> None:
        self._edit = edit

        self.run = to_streamed_response_wrapper(
            edit.run,
        )
        self.run_job = to_streamed_response_wrapper(
            edit.run_job,
        )


class AsyncEditResourceWithStreamingResponse:
    def __init__(self, edit: AsyncEditResource) -> None:
        self._edit = edit

        self.run = async_to_streamed_response_wrapper(
            edit.run,
        )
        self.run_job = async_to_streamed_response_wrapper(
            edit.run_job,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["ExtractUsage"]


class ExtractUsage(BaseModel):
    num_fields: int

    num_pages: int

    credits: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["LargeTableChunkingConfig"]


class LargeTableChunkingConfig(BaseModel):
    enabled: Optional[bool] = None
    """
    If large tables should be chunked into smaller tables, currently only supported
    on spreadsheet and CSV files.
    """

    size: Optional[int] = None
    """The max row/column size for a table to be chunked.

    Defaults to 50. Header rows/columns are persisted based on heuristics.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["EnrichConfig"]


class EnrichConfig(BaseModel):
    enabled: Optional[bool] = None
    """
    If enabled, a large language/vision model will be used to postprocess the
    extracted content. Note: enabling enrich requires tables be outputted in
    markdown format. Defaults to False.
    """

    mode: Optional[Literal["standard", "page", "table"]] = None
    """The mode to use for enrichment. Defaults to standard"""

    prompt: Optional[str] = None
    """Add information to the prompt for enrichment."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias, TypedDict

__all__ = ["SpreadsheetParam", "SplitLargeTables", "SplitLargeTablesSize", "SplitLargeTablesSizeSplitLargeTableSizes"]


class SplitLargeTablesSizeSplitLargeTableSizes(TypedDict, total=False):
    column: Optional[int]
    """The number of columns to include in each chunk when splitting large tables.

    Does not chunk columns if set to None.
    """

    row: Optional[int]
    """The number of rows to include in each chunk when splitting large tables.

    Does not chunk rows if set to None.
    """


SplitLargeTablesSize: TypeAlias = Union[int, SplitLargeTablesSizeSplitLargeTableSizes]


class SplitLargeTables(TypedDict, total=False):
    enabled: bool
    """If True, split large tables into smaller tables. Defaults to True."""

    size: SplitLargeTablesSize
    """The size of the tables to split into.

    Defaults to 50. Use 'row' and 'column' to independently specify the number of
    rows and columns to include when splitting. If you only want to split by rows or
    columns, set the other value to None.
    """


class SpreadsheetParam(TypedDict, total=False):
    clustering: Literal["accurate", "fast", "disabled"]
    """
    In a spreadsheet with different tables inside, we enable splitting up the tables
    by default. Accurate mode applies more powerful models for superior accuracy, at
    5× the default per-cell rate. Disabling will register as one large table.
    """

    exclude: List[Literal["hidden_sheets", "hidden_rows", "hidden_cols", "styling", "spreadsheet_images"]]
    """Whether to exclude hidden sheets, rows, or columns in the output."""

    include: List[Literal["cell_colors", "formula", "dropdowns"]]
    """Whether to include cell color, formula, and dropdown information in the output."""

    split_large_tables: SplitLargeTables



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .shared_params.upload import Upload
from .pipeline_settings_param import PipelineSettingsParam

__all__ = ["PipelineRunParams", "Input"]


class PipelineRunParams(TypedDict, total=False):
    input: Required[Input]
    """For parse/split/extract pipelines, the URL of the document to be processed.

    You can provide one of the following: 1. A publicly available URL 2. A presigned
    S3 URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
    directly uploading a document 4. A jobid:// prefixed URL obtained from a
    previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
    API only)

                For edit pipelines, this should be a string containing the edit instructions
    """

    pipeline_id: Required[str]
    """The ID of the pipeline to use for the document."""

    settings: PipelineSettingsParam
    """Settings for pipeline execution that override pipeline defaults."""


Input: TypeAlias = Union[str, SequenceNotStr[str], Upload]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel
from .parse_usage import ParseUsage
from .bounding_box import BoundingBox

__all__ = [
    "ParseResponse",
    "Result",
    "ResultFullResult",
    "ResultFullResultChunk",
    "ResultFullResultChunkBlock",
    "ResultFullResultChunkBlockGranularConfidence",
    "ResultFullResultOcr",
    "ResultFullResultOcrLine",
    "ResultFullResultOcrWord",
    "ResultURLResult",
]


class ResultFullResultChunkBlockGranularConfidence(BaseModel):
    """Granular confidence scores for the block.

    It is a dictionary of confidence scores for the block. The confidence scores will not be None if the user has enabled numeric confidence scores.
    """

    extract_confidence: Optional[float] = None

    parse_confidence: Optional[float] = None


class ResultFullResultChunkBlock(BaseModel):
    bbox: BoundingBox
    """The bounding box of the block extracted from the document."""

    content: str
    """The content of the block extracted from the document."""

    type: Literal[
        "Header",
        "Footer",
        "Title",
        "Section Header",
        "Page Number",
        "List Item",
        "Figure",
        "Table",
        "Key Value",
        "Text",
        "Comment",
        "Signature",
    ]
    """The type of block extracted from the document."""

    chart_data: Optional[List[str]] = None
    """
    (Experimental) The URL/link to chart data JSON for figure blocks processed by
    chart agent.
    """

    confidence: Optional[str] = None
    """The confidence for the block.

    It is either low or high and takes into account factors like OCR and table
    structure
    """

    extra: Optional[Dict[str, object]] = None
    """Extra metadata fields for the block.

    Fields like 'is_chart' will only appear when set to True.
    """

    granular_confidence: Optional[ResultFullResultChunkBlockGranularConfidence] = None
    """Granular confidence scores for the block.

    It is a dictionary of confidence scores for the block. The confidence scores
    will not be None if the user has enabled numeric confidence scores.
    """

    image_url: Optional[str] = None
    """(Experimental) The URL of the image associated with the block."""


class ResultFullResultChunk(BaseModel):
    blocks: List[ResultFullResultChunkBlock]

    content: str
    """The content of the chunk extracted from the document."""

    embed: str
    """Chunk content optimized for embedding and retrieval."""

    enriched: Optional[str] = None
    """The enriched content of the chunk extracted from the document."""

    enrichment_success: Optional[bool] = None
    """Whether the enrichment was successful."""


class ResultFullResultOcrLine(BaseModel):
    bbox: BoundingBox

    text: str

    chunk_index: Optional[int] = None
    """The index of the chunk that the line belongs to."""

    confidence: Optional[float] = None
    """OCR confidence score between 0 and 1, where 1 indicates highest confidence"""

    rotation: Optional[int] = None
    """The rotation angle in degrees, from 0 to 360, counterclockwise."""


class ResultFullResultOcrWord(BaseModel):
    bbox: BoundingBox

    text: str

    chunk_index: Optional[int] = None
    """The index of the chunk that the word belongs to."""

    confidence: Optional[float] = None
    """OCR confidence score between 0 and 1, where 1 indicates highest confidence"""

    rotation: Optional[int] = None
    """The rotation angle in degrees, from 0 to 360, counterclockwise."""


class ResultFullResultOcr(BaseModel):
    lines: List[ResultFullResultOcrLine]

    words: List[ResultFullResultOcrWord]


class ResultFullResult(BaseModel):
    chunks: List[ResultFullResultChunk]

    type: Literal["full"]
    """type = 'full'"""

    custom: Optional[object] = None

    ocr: Optional[ResultFullResultOcr] = None


class ResultURLResult(BaseModel):
    result_id: str

    type: Literal["url"]
    """type = 'url'"""

    url: str


Result: TypeAlias = Union[ResultFullResult, ResultURLResult]


class ParseResponse(BaseModel):
    duration: float
    """The duration of the parse request in seconds."""

    job_id: str

    result: Result
    """The response from the document processing service.

    Note that there can be two types of responses, Full Result and URL Result. This
    is due to limitations on the max return size on HTTPS. If the response is too
    large, it will be returned as a presigned URL in the URL response. You should
    handle this in your application.
    """

    usage: ParseUsage

    pdf_url: Optional[str] = None
    """The storage URL of the converted PDF file."""

    studio_link: Optional[str] = None
    """The link to the studio pipeline for the document."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from .._models import BaseModel

__all__ = ["ParseUsage"]


class ParseUsage(BaseModel):
    num_pages: int

    credit_breakdown: Optional[Dict[str, float]] = None

    credits: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["BoundingBox"]


class BoundingBox(BaseModel):
    height: float

    left: float

    page: int
    """The page number of the bounding box (1-indexed)."""

    top: float

    width: float

    original_page: Optional[int] = None
    """The page number in the original document of the bounding box (1-indexed)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .enhance import Enhance
from .settings import Settings
from ..._models import BaseModel
from .retrieval import Retrieval
from .formatting import Formatting
from .spreadsheet import Spreadsheet

__all__ = ["ParseOptions"]


class ParseOptions(BaseModel):
    enhance: Optional[Enhance] = None

    formatting: Optional[Formatting] = None

    retrieval: Optional[Retrieval] = None

    settings: Optional[Settings] = None

    spreadsheet: Optional[Spreadsheet] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from reducto import Reducto, AsyncReducto
from tests.utils import assert_matches_type
from reducto.types import (
    ExtractRunResponse,
    AsyncExtractResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestExtract:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_overload_1(self, client: Reducto) -> None:
        extract = client.extract.run(
            input="string",
        )
        assert_matches_type(ExtractRunResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_with_all_params_overload_1(self, client: Reducto) -> None:
        extract = client.extract.run(
            input="string",
            instructions={
                "schema": {},
                "system_prompt": "system_prompt",
            },
            parsing={
                "enhance": {
                    "agentic": [
                        {
                            "scope": "table",
                            "prompt": "prompt",
                        }
                    ],
                    "intelligent_ordering": True,
                    "summarize_figures": True,
                },
                "formatting": {
                    "add_page_markers": True,
                    "include": ["change_tracking"],
                    "merge_tables": True,
                    "table_output_format": "html",
                },
                "retrieval": {
                    "chunking": {
                        "chunk_mode": "variable",
                        "chunk_overlap": 0,
                        "chunk_size": 0,
                    },
                    "embedding_optimized": True,
                    "filter_blocks": ["Header"],
                },
                "settings": {
                    "document_password": "document_password",
                    "embed_pdf_metadata": True,
                    "extraction_mode": "ocr",
                    "force_file_extension": "force_file_extension",
                    "force_url_result": True,
                    "ocr_system": "standard",
                    "page_range": {
                        "end": 0,
                        "start": 0,
                    },
                    "persist_results": True,
                    "return_images": ["figure"],
                    "return_ocr_data": True,
                    "timeout": 0,
                },
                "spreadsheet": {
                    "clustering": "accurate",
                    "exclude": ["hidden_sheets"],
                    "include": ["cell_colors"],
                    "split_large_tables": {
                        "enabled": True,
                        "size": 0,
                    },
                },
            },
            settings={
                "array_extract": True,
                "citations": {
                    "enabled": True,
                    "numerical_confidence": True,
                },
                "deep_extract": True,
                "include_images": True,
                "optimize_for_latency": True,
            },
        )
        assert_matches_type(ExtractRunResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_run_overload_1(self, client: Reducto) -> None:
        response = client.extract.with_raw_response.run(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        extract = response.parse()
        assert_matches_type(ExtractRunResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_run_overload_1(self, client: Reducto) -> None:
        with client.extract.with_streaming_response.run(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            extract = response.parse()
            assert_matches_type(ExtractRunResponse, extract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_overload_2(self, client: Reducto) -> None:
        extract = client.extract.run(
            input="string",
        )
        assert_matches_type(ExtractRunResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_with_all_params_overload_2(self, client: Reducto) -> None:
        extract = client.extract.run(
            input="string",
            async_={
                "metadata": {},
                "priority": True,
                "webhook": {
                    "channels": ["string"],
                    "mode": "svix",
                },
            },
            instructions={
                "schema": {},
                "system_prompt": "system_prompt",
            },
            parsing={
                "enhance": {
                    "agentic": [
                        {
                            "scope": "table",
                            "prompt": "prompt",
                        }
                    ],
                    "intelligent_ordering": True,
                    "summarize_figures": True,
                },
                "formatting": {
                    "add_page_markers": True,
                    "include": ["change_tracking"],
                    "merge_tables": True,
                    "table_output_format": "html",
                },
                "retrieval": {
                    "chunking": {
                        "chunk_mode": "variable",
                        "chunk_overlap": 0,
                        "chunk_size": 0,
                    },
                    "embedding_optimized": True,
                    "filter_blocks": ["Header"],
                },
                "settings": {
                    "document_password": "document_password",
                    "embed_pdf_metadata": True,
                    "extraction_mode": "ocr",
                    "force_file_extension": "force_file_extension",
                    "force_url_result": True,
                    "ocr_system": "standard",
                    "page_range": {
                        "end": 0,
                        "start": 0,
                    },
                    "persist_results": True,
                    "return_images": ["figure"],
                    "return_ocr_data": True,
                    "timeout": 0,
                },
                "spreadsheet": {
                    "clustering": "accurate",
                    "exclude": ["hidden_sheets"],
                    "include": ["cell_colors"],
                    "split_large_tables": {
                        "enabled": True,
                        "size": 0,
                    },
                },
            },
            settings={
                "array_extract": True,
                "citations": {
                    "enabled": True,
                    "numerical_confidence": True,
                },
                "deep_extract": True,
                "include_images": True,
                "optimize_for_latency": True,
            },
        )
        assert_matches_type(ExtractRunResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_run_overload_2(self, client: Reducto) -> None:
        response = client.extract.with_raw_response.run(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        extract = response.parse()
        assert_matches_type(ExtractRunResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_run_overload_2(self, client: Reducto) -> None:
        with client.extract.with_streaming_response.run(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            extract = response.parse()
            assert_matches_type(ExtractRunResponse, extract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_job(self, client: Reducto) -> None:
        extract = client.extract.run_job(
            input="string",
        )
        assert_matches_type(AsyncExtractResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_job_with_all_params(self, client: Reducto) -> None:
        extract = client.extract.run_job(
            input="string",
            async_={
                "metadata": {},
                "priority": True,
                "webhook": {
                    "channels": ["string"],
                    "mode": "svix",
                },
            },
            instructions={
                "schema": {},
                "system_prompt": "system_prompt",
            },
            parsing={
                "enhance": {
                    "agentic": [
                        {
                            "scope": "table",
                            "prompt": "prompt",
                        }
                    ],
                    "intelligent_ordering": True,
                    "summarize_figures": True,
                },
                "formatting": {
                    "add_page_markers": True,
                    "include": ["change_tracking"],
                    "merge_tables": True,
                    "table_output_format": "html",
                },
                "retrieval": {
                    "chunking": {
                        "chunk_mode": "variable",
                        "chunk_overlap": 0,
                        "chunk_size": 0,
                    },
                    "embedding_optimized": True,
                    "filter_blocks": ["Header"],
                },
                "settings": {
                    "document_password": "document_password",
                    "embed_pdf_metadata": True,
                    "extraction_mode": "ocr",
                    "force_file_extension": "force_file_extension",
                    "force_url_result": True,
                    "ocr_system": "standard",
                    "page_range": {
                        "end": 0,
                        "start": 0,
                    },
                    "persist_results": True,
                    "return_images": ["figure"],
                    "return_ocr_data": True,
                    "timeout": 0,
                },
                "spreadsheet": {
                    "clustering": "accurate",
                    "exclude": ["hidden_sheets"],
                    "include": ["cell_colors"],
                    "split_large_tables": {
                        "enabled": True,
                        "size": 0,
                    },
                },
            },
            settings={
                "array_extract": True,
                "citations": {
                    "enabled": True,
                    "numerical_confidence": True,
                },
                "deep_extract": True,
                "include_images": True,
                "optimize_for_latency": True,
            },
        )
        assert_matches_type(AsyncExtractResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_run_job(self, client: Reducto) -> None:
        response = client.extract.with_raw_response.run_job(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        extract = response.parse()
        assert_matches_type(AsyncExtractResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_run_job(self, client: Reducto) -> None:
        with client.extract.with_streaming_response.run_job(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            extract = response.parse()
            assert_matches_type(AsyncExtractResponse, extract, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncExtract:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_overload_1(self, async_client: AsyncReducto) -> None:
        extract = await async_client.extract.run(
            input="string",
        )
        assert_matches_type(ExtractRunResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_with_all_params_overload_1(self, async_client: AsyncReducto) -> None:
        extract = await async_client.extract.run(
            input="string",
            instructions={
                "schema": {},
                "system_prompt": "system_prompt",
            },
            parsing={
                "enhance": {
                    "agentic": [
                        {
                            "scope": "table",
                            "prompt": "prompt",
                        }
                    ],
                    "intelligent_ordering": True,
                    "summarize_figures": True,
                },
                "formatting": {
                    "add_page_markers": True,
                    "include": ["change_tracking"],
                    "merge_tables": True,
                    "table_output_format": "html",
                },
                "retrieval": {
                    "chunking": {
                        "chunk_mode": "variable",
                        "chunk_overlap": 0,
                        "chunk_size": 0,
                    },
                    "embedding_optimized": True,
                    "filter_blocks": ["Header"],
                },
                "settings": {
                    "document_password": "document_password",
                    "embed_pdf_metadata": True,
                    "extraction_mode": "ocr",
                    "force_file_extension": "force_file_extension",
                    "force_url_result": True,
                    "ocr_system": "standard",
                    "page_range": {
                        "end": 0,
                        "start": 0,
                    },
                    "persist_results": True,
                    "return_images": ["figure"],
                    "return_ocr_data": True,
                    "timeout": 0,
                },
                "spreadsheet": {
                    "clustering": "accurate",
                    "exclude": ["hidden_sheets"],
                    "include": ["cell_colors"],
                    "split_large_tables": {
                        "enabled": True,
                        "size": 0,
                    },
                },
            },
            settings={
                "array_extract": True,
                "citations": {
                    "enabled": True,
                    "numerical_confidence": True,
                },
                "deep_extract": True,
                "include_images": True,
                "optimize_for_latency": True,
            },
        )
        assert_matches_type(ExtractRunResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_run_overload_1(self, async_client: AsyncReducto) -> None:
        response = await async_client.extract.with_raw_response.run(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        extract = await response.parse()
        assert_matches_type(ExtractRunResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_run_overload_1(self, async_client: AsyncReducto) -> None:
        async with async_client.extract.with_streaming_response.run(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            extract = await response.parse()
            assert_matches_type(ExtractRunResponse, extract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_overload_2(self, async_client: AsyncReducto) -> None:
        extract = await async_client.extract.run(
            input="string",
        )
        assert_matches_type(ExtractRunResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_with_all_params_overload_2(self, async_client: AsyncReducto) -> None:
        extract = await async_client.extract.run(
            input="string",
            async_={
                "metadata": {},
                "priority": True,
                "webhook": {
                    "channels": ["string"],
                    "mode": "svix",
                },
            },
            instructions={
                "schema": {},
                "system_prompt": "system_prompt",
            },
            parsing={
                "enhance": {
                    "agentic": [
                        {
                            "scope": "table",
                            "prompt": "prompt",
                        }
                    ],
                    "intelligent_ordering": True,
                    "summarize_figures": True,
                },
                "formatting": {
                    "add_page_markers": True,
                    "include": ["change_tracking"],
                    "merge_tables": True,
                    "table_output_format": "html",
                },
                "retrieval": {
                    "chunking": {
                        "chunk_mode": "variable",
                        "chunk_overlap": 0,
                        "chunk_size": 0,
                    },
                    "embedding_optimized": True,
                    "filter_blocks": ["Header"],
                },
                "settings": {
                    "document_password": "document_password",
                    "embed_pdf_metadata": True,
                    "extraction_mode": "ocr",
                    "force_file_extension": "force_file_extension",
                    "force_url_result": True,
                    "ocr_system": "standard",
                    "page_range": {
                        "end": 0,
                        "start": 0,
                    },
                    "persist_results": True,
                    "return_images": ["figure"],
                    "return_ocr_data": True,
                    "timeout": 0,
                },
                "spreadsheet": {
                    "clustering": "accurate",
                    "exclude": ["hidden_sheets"],
                    "include": ["cell_colors"],
                    "split_large_tables": {
                        "enabled": True,
                        "size": 0,
                    },
                },
            },
            settings={
                "array_extract": True,
                "citations": {
                    "enabled": True,
                    "numerical_confidence": True,
                },
                "deep_extract": True,
                "include_images": True,
                "optimize_for_latency": True,
            },
        )
        assert_matches_type(ExtractRunResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_run_overload_2(self, async_client: AsyncReducto) -> None:
        response = await async_client.extract.with_raw_response.run(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        extract = await response.parse()
        assert_matches_type(ExtractRunResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_run_overload_2(self, async_client: AsyncReducto) -> None:
        async with async_client.extract.with_streaming_response.run(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            extract = await response.parse()
            assert_matches_type(ExtractRunResponse, extract, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_job(self, async_client: AsyncReducto) -> None:
        extract = await async_client.extract.run_job(
            input="string",
        )
        assert_matches_type(AsyncExtractResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_job_with_all_params(self, async_client: AsyncReducto) -> None:
        extract = await async_client.extract.run_job(
            input="string",
            async_={
                "metadata": {},
                "priority": True,
                "webhook": {
                    "channels": ["string"],
                    "mode": "svix",
                },
            },
            instructions={
                "schema": {},
                "system_prompt": "system_prompt",
            },
            parsing={
                "enhance": {
                    "agentic": [
                        {
                            "scope": "table",
                            "prompt": "prompt",
                        }
                    ],
                    "intelligent_ordering": True,
                    "summarize_figures": True,
                },
                "formatting": {
                    "add_page_markers": True,
                    "include": ["change_tracking"],
                    "merge_tables": True,
                    "table_output_format": "html",
                },
                "retrieval": {
                    "chunking": {
                        "chunk_mode": "variable",
                        "chunk_overlap": 0,
                        "chunk_size": 0,
                    },
                    "embedding_optimized": True,
                    "filter_blocks": ["Header"],
                },
                "settings": {
                    "document_password": "document_password",
                    "embed_pdf_metadata": True,
                    "extraction_mode": "ocr",
                    "force_file_extension": "force_file_extension",
                    "force_url_result": True,
                    "ocr_system": "standard",
                    "page_range": {
                        "end": 0,
                        "start": 0,
                    },
                    "persist_results": True,
                    "return_images": ["figure"],
                    "return_ocr_data": True,
                    "timeout": 0,
                },
                "spreadsheet": {
                    "clustering": "accurate",
                    "exclude": ["hidden_sheets"],
                    "include": ["cell_colors"],
                    "split_large_tables": {
                        "enabled": True,
                        "size": 0,
                    },
                },
            },
            settings={
                "array_extract": True,
                "citations": {
                    "enabled": True,
                    "numerical_confidence": True,
                },
                "deep_extract": True,
                "include_images": True,
                "optimize_for_latency": True,
            },
        )
        assert_matches_type(AsyncExtractResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_run_job(self, async_client: AsyncReducto) -> None:
        response = await async_client.extract.with_raw_response.run_job(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        extract = await response.parse()
        assert_matches_type(AsyncExtractResponse, extract, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_run_job(self, async_client: AsyncReducto) -> None:
        async with async_client.extract.with_streaming_response.run_job(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            extract = await response.parse()
            assert_matches_type(AsyncExtractResponse, extract, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from .parse_response import ParseResponse
from .async_parse_response import AsyncParseResponse

__all__ = ["ParseRunResponse"]

ParseRunResponse: TypeAlias = Union[ParseResponse, AsyncParseResponse]



from typing import Optional

import httpx
from pydantic import BaseModel

from reducto.types import ParseResponse
from reducto.types.shared.parse_response import ParseUsage, ResultFullResult

__all__ = ["FullParseResponse", "handle_url_response", "ParseResponse"]


class FullParseResponse(BaseModel):
    duration: float
    """The duration of the parse request in seconds."""

    job_id: str

    result: ResultFullResult
    """The full response from the document processing service."""

    usage: ParseUsage

    pdf_url: Optional[str] = None
    """The storage URL of the converted PDF file."""


def handle_url_response(response: ParseResponse) -> FullParseResponse:
    if response.result.type == "url":
        with httpx.stream("GET", response.result.url) as r:
            content = r.read().decode()
            result = ResultFullResult.model_validate_json(content)
            return FullParseResponse(
                duration=response.duration,
                job_id=response.job_id,
                result=result,
                usage=response.usage,
            )
    else:
        return FullParseResponse.model_validate(response.model_dump())



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from ..._types import SequenceNotStr

__all__ = ["WebhookConfigNew"]


class WebhookConfigNew(TypedDict, total=False):
    channels: SequenceNotStr[str]
    """
    A list of Svix channels the message will be delivered down, omit to send to all
    channels.
    """

    metadata: object
    """JSON metadata included in webhook request body"""

    mode: Literal["disabled", "svix", "direct"]
    """The mode to use for webhook delivery.

    Defaults to 'disabled'. We recommend using 'svix' for production environments.
    """

    url: str
    """The URL to send the webhook to (if using direct webhoook)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from ..._types import SequenceNotStr

__all__ = ["SvixWebhookConfig"]


class SvixWebhookConfig(TypedDict, total=False):
    channels: SequenceNotStr[str]
    """
    A list of Svix channels the message will be delivered down, omit to send to all
    channels.
    """

    mode: Literal["svix"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import TypeAlias

from ..._models import BaseModel

__all__ = ["SplitLargeTables", "Size", "SizeSplitLargeTableSizes"]


class SizeSplitLargeTableSizes(BaseModel):
    column: Optional[int] = None
    """The number of columns to include in each chunk when splitting large tables.

    Does not chunk columns if set to None.
    """

    row: Optional[int] = None
    """The number of rows to include in each chunk when splitting large tables.

    Does not chunk rows if set to None.
    """


Size: TypeAlias = Union[int, SizeSplitLargeTableSizes]


class SplitLargeTables(BaseModel):
    enabled: Optional[bool] = None
    """If True, split large tables into smaller tables. Defaults to True."""

    size: Optional[Size] = None
    """The size of the tables to split into.

    Defaults to 50. Use 'row' and 'column' to independently specify the number of
    rows and columns to include when splitting. If you only want to split by rows or
    columns, set the other value to None.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SvixWebhookConfig"]


class SvixWebhookConfig(BaseModel):
    channels: Optional[List[str]] = None
    """
    A list of Svix channels the message will be delivered down, omit to send to all
    channels.
    """

    mode: Optional[Literal["svix"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from typing_extensions import Literal, TypeAlias, TypedDict

from . import page_range
from ..._types import SequenceNotStr

__all__ = ["Settings", "PageRange"]

PageRange: TypeAlias = Union[page_range.PageRange, Iterable[page_range.PageRange], Iterable[int], SequenceNotStr[str]]


class Settings(TypedDict, total=False):
    document_password: Optional[str]
    """Password to decrypt password-protected documents."""

    embed_pdf_metadata: bool
    """If True, embed OCR metadata into the returned PDF. Defaults to False."""

    extraction_mode: Literal["ocr", "hybrid"]
    """The mode to use for text extraction from PDFs.

    OCR mode uses optical character recognition only. Hybrid mode combines OCR with
    embedded PDF text for best accuracy (default).
    """

    force_file_extension: Optional[str]
    """Force the URL to be downloaded as a specific file extension (e.g. `.png`)."""

    force_url_result: bool
    """Force the result to be returned in URL form."""

    ocr_system: Literal["standard", "legacy"]
    """Standard is our best multilingual OCR system.

    Legacy only supports germanic languages and is available for backwards
    compatibility.
    """

    page_range: Optional[PageRange]
    """The page range to process (1-indexed).

    By default, the entire document is processed. For spreadsheets, you can also
    provide a list of sheet names.
    """

    persist_results: bool
    """If True, persist the results indefinitely. Defaults to False."""

    return_images: List[Literal["figure", "table"]]
    """Whether to return images for the specified block types.

    By default, no images are returned.
    """

    return_ocr_data: bool
    """If True, return OCR data in the result. Defaults to False."""

    timeout: Optional[float]
    """The timeout for the job in seconds."""
